@@ -6,13 +6,11 @@
 : "${JSONC_INSTALL:=$LOCAL_INSTALLS/jsonc-install}"
 : "${GPG_INSTALL:=$LOCAL_INSTALLS/gpg-install}"
 : "${RNP_INSTALL:=$LOCAL_INSTALLS/rnp-install}"
-: "${RUBY_RNP_INSTALL:=$LOCAL_INSTALLS/ruby-rnp}"
-: "${RUBY_RNP_VERSION:=main}"
 : "${CPU:=}"
 : "${SUDO:=}"
 
 for var in LOCAL_BUILDS LOCAL_INSTALLS BOTAN_INSTALL JSONC_INSTALL \
-  GPG_INSTALL RNP_INSTALL RUBY_RNP_INSTALL RUBY_RNP_VERSION CPU SUDO; do
+  GPG_INSTALL RNP_INSTALL CPU SUDO; do
   export "${var?}"
 done
 
@@ -26,3 +24,6 @@ fi
 BOTAN_MODULES=$(<ci/botan-modules tr '\n' ',')
 
 export BOTAN_MODULES
+
+# Don't clean up tempdirs when in CI runners to save time. Unset to disable.
+export RNP_KEEP_TEMP=1
