@@ -32,24 +32,9 @@ install_botan() {
     # but it makes no sense and is not supported for Windows
     local extra_cflags="-fPIC"
     case "${OS}" in
-      msys)
-        osparam=(--os=mingw)
-        run=python
-        # Just get rid of all newlines!
-        BOTAN_MODULES="${BOTAN_MODULES//$'\r\n'/}"
-        BOTAN_MODULES="${BOTAN_MODULES//$'\r'/}"
-        BOTAN_MODULES="${BOTAN_MODULES//$'\n'/}"
-
-        # Deal with "error: ignoring '#pragma comment"
-        extra_cflags="-Wno-error=unknown-pragmas"
-        # Drop -fPIC
-        # Expecting implicit --cc-bin=g++/clang++ due to environment variable CXX
-        # Expecting implicit --cxxflags=-I/clang64/include  due to environment variable CXXFLAGS
-        # Expecting implicit --ldflags=-L<runner root>/clang64/lib -lomp due to environment variable LDFLAGS
-        ;;
       linux)
         case "${DIST_VERSION}" in
-          centos-8|fedora-*|debian-*)
+          centos-8|centos-9|fedora-*|debian-*)
             run=run_in_python_venv
             ;;
         esac
@@ -168,6 +153,7 @@ _install_gpg() {
       "--disable-pinentry-tqt"
       "--disable-pinentry-fltk"
       "--enable-maintainer-mode"
+      "--enable-install-gpg-error-config"
       ${cpuparam+"${cpuparam[@]}"}
     )
 
@@ -221,7 +207,6 @@ _install_gpg() {
 
 
 install_gpg() {
-  # gpg - for msys/windows we use shipped gpg2 version
   local gpg_build=${LOCAL_BUILDS}/gpg
 
   # shellcheck disable=SC2153
@@ -233,8 +218,11 @@ install_gpg() {
     case "${GPG_VERSION}" in
       stable)
         #                              npth libgpg-error libgcrypt libassuan libksba pinentry gnupg
-        _install_gpg component-version 1.6  1.43         1.8.8     2.5.5     1.6.0   1.1.0    2.2.33
-        # _install_gpg component-version 1.6  1.42         1.9.2     2.5.5     1.5.0   1.1.1    2.2.27
+        _install_gpg component-version 1.6  1.46         1.10.1     2.5.5     1.6.3  1.2.1    2.4.0
+        ;;
+      lts)
+        #                              npth libgpg-error libgcrypt libassuan libksba pinentry gnupg
+        _install_gpg component-version 1.6  1.46         1.8.10    2.5.5     1.6.3   1.2.1    2.2.41
         ;;
       beta)
         #                              npth    libgpg-error libgcrypt libassuan libksba pinentry gnupg
