@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2017 Ribose Inc.
+ * Copyright (c) 2017-2022 Ribose Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,121 +26,14 @@
 
 #include <string.h>
 #include <botan/ffi.h>
+#include "hash_botan.hpp"
 #include "ecdh.h"
-#include "hash.h"
+#include "ecdh_utils.h"
 #include "symmetric.h"
 #include "types.h"
 #include "utils.h"
 #include "mem.h"
-
-#define MAX_SP800_56A_OTHER_INFO 56
-// Keys up to 312 bits (+1 bytes of PKCS5 padding)
-#define MAX_SESSION_KEY_SIZE 40
-
-/* Used by ECDH keys. Specifies which hash and wrapping algorithm
- * to be used (see point 15. of RFC 4880).
- *
- * Note: sync with ec_curves.
- */
-static const struct ecdh_params_t {
-    pgp_curve_t    curve;    /* Curve ID */
-    pgp_hash_alg_t hash;     /* Hash used by kdf */
-    pgp_symm_alg_t wrap_alg; /* Symmetric algorithm used to wrap KEK*/
-} ecdh_params[] = {
-  {.curve = PGP_CURVE_NIST_P_256, .hash = PGP_HASH_SHA256, .wrap_alg = PGP_SA_AES_128},
-  {.curve = PGP_CURVE_NIST_P_384, .hash = PGP_HASH_SHA384, .wrap_alg = PGP_SA_AES_192},
-  {.curve = PGP_CURVE_NIST_P_521, .hash = PGP_HASH_SHA512, .wrap_alg = PGP_SA_AES_256},
-  {.curve = PGP_CURVE_BP256, .hash = PGP_HASH_SHA256, .wrap_alg = PGP_SA_AES_128},
-  {.curve = PGP_CURVE_BP384, .hash = PGP_HASH_SHA384, .wrap_alg = PGP_SA_AES_192},
-  {.curve = PGP_CURVE_BP512, .hash = PGP_HASH_SHA512, .wrap_alg = PGP_SA_AES_256},
-  {.curve = PGP_CURVE_25519, .hash = PGP_HASH_SHA256, .wrap_alg = PGP_SA_AES_128},
-  {.curve = PGP_CURVE_P256K1, .hash = PGP_HASH_SHA256, .wrap_alg = PGP_SA_AES_128},
-};
-
-// "Anonymous Sender " in hex
-static const unsigned char ANONYMOUS_SENDER[] = {0x41, 0x6E, 0x6F, 0x6E, 0x79, 0x6D, 0x6F,
-                                                 0x75, 0x73, 0x20, 0x53, 0x65, 0x6E, 0x64,
-                                                 0x65, 0x72, 0x20, 0x20, 0x20, 0x20};
-
-// returns size of data written to other_info
-static size_t
-kdf_other_info_serialize(uint8_t                  other_info[MAX_SP800_56A_OTHER_INFO],
-                         const ec_curve_desc_t *  ec_curve,
-                         const pgp_fingerprint_t &fingerprint,
-                         const pgp_hash_alg_t     kdf_hash,
-                         const pgp_symm_alg_t     wrap_alg)
-{
-    if (fingerprint.length < 20) {
-        RNP_LOG("Implementation error: unexpected fingerprint length");
-        return false;
-    }
-
-    uint8_t *buf_ptr = &other_info[0];
-
-    /* KDF-OtherInfo: AlgorithmID
-     *   Current implementation will always use SHA-512 and AES-256 for KEK wrapping
-     */
-    *(buf_ptr++) = ec_curve->OIDhex_len;
-    memcpy(buf_ptr, ec_curve->OIDhex, ec_curve->OIDhex_len);
-    buf_ptr += ec_curve->OIDhex_len;
-    *(buf_ptr++) = PGP_PKA_ECDH;
-    // size of following 3 params (each 1 byte)
-    *(buf_ptr++) = 0x03;
-    // Value reserved for future use
-    *(buf_ptr++) = 0x01;
-    // Hash used with KDF
-    *(buf_ptr++) = kdf_hash;
-    // Algorithm ID used for key wrapping
-    *(buf_ptr++) = wrap_alg;
-
-    /* KDF-OtherInfo: PartyUInfo
-     *   20 bytes representing "Anonymous Sender "
-     */
-    memcpy(buf_ptr, ANONYMOUS_SENDER, sizeof(ANONYMOUS_SENDER));
-
-    buf_ptr += sizeof(ANONYMOUS_SENDER);
-
-    // keep 20, as per spec
-    memcpy(buf_ptr, fingerprint.fingerprint, 20);
-    return (buf_ptr - other_info) + 20 /*anonymous_sender*/;
-}
-
-static bool
-pad_pkcs7(uint8_t *buf, size_t buf_len, size_t offset)
-{
-    if (buf_len <= offset) {
-        // Must have at least 1 byte of padding
-        return false;
-    }
-
-    const uint8_t pad_byte = buf_len - offset;
-    memset(buf + offset, pad_byte, pad_byte);
-    return true;
-}
-
-static bool
-unpad_pkcs7(uint8_t *buf, size_t buf_len, size_t *offset)
-{
-    if (!buf || !offset || !buf_len) {
-        return false;
-    }
-
-    uint8_t        err = 0;
-    const uint8_t  pad_byte = buf[buf_len - 1];
-    const uint32_t pad_begin = buf_len - pad_byte;
-
-    // TODO: Still >, <, and <=,==  are not constant time (maybe?)
-    err |= (pad_byte > buf_len);
-    err |= (pad_byte == 0);
-
-    /* Check if padding is OK */
-    for (size_t c = 0; c < buf_len; c++) {
-        err |= (buf[c] ^ pad_byte) * (pad_begin <= c);
-    }
-
-    *offset = pad_begin;
-    return (err == 0);
-}
+#include "bn.h"
 
 // Produces kek of size kek_len which corresponds to length of wrapping key
 static bool
@@ -176,27 +69,14 @@ compute_kek(uint8_t *              kek,
         goto end;
     }
 
-    snprintf(kdf_name, sizeof(kdf_name), "SP800-56A(%s)", pgp_hash_name_botan(hash_alg));
+    snprintf(
+      kdf_name, sizeof(kdf_name), "SP800-56A(%s)", rnp::Hash_Botan::name_backend(hash_alg));
     ret = !botan_kdf(
       kdf_name, kek, kek_len, s.data(), s_len, NULL, 0, other_info, other_info_size);
 end:
     return ret && !botan_pk_op_key_agreement_destroy(op_key_agreement);
 }
 
-bool
-ecdh_set_params(pgp_ec_key_t *key, pgp_curve_t curve_id)
-{
-    for (size_t i = 0; i < ARRAY_SIZE(ecdh_params); i++) {
-        if (ecdh_params[i].curve == curve_id) {
-            key->kdf_hash_alg = ecdh_params[i].hash;
-            key->key_wrap_alg = ecdh_params[i].wrap_alg;
-            return true;
-        }
-    }
-
-    return false;
-}
-
 static bool
 ecdh_load_public_key(botan_pubkey_t *pubkey, const pgp_ec_key_t *key)
 {
@@ -273,7 +153,7 @@ ecdh_load_secret_key(botan_privkey_t *seckey, const pgp_ec_key_t *key)
 }
 
 rnp_result_t
-ecdh_validate_key(rng_t *rng, const pgp_ec_key_t *key, bool secret)
+ecdh_validate_key(rnp::RNG *rng, const pgp_ec_key_t *key, bool secret)
 {
     botan_pubkey_t  bpkey = NULL;
     botan_privkey_t bskey = NULL;
@@ -285,7 +165,7 @@ ecdh_validate_key(rng_t *rng, const pgp_ec_key_t *key, bool secret)
     }
 
     if (!ecdh_load_public_key(&bpkey, key) ||
-        botan_pubkey_check_key(bpkey, rng_handle(rng), 0)) {
+        botan_pubkey_check_key(bpkey, rng->handle(), 0)) {
         goto done;
     }
     if (!secret) {
@@ -294,7 +174,7 @@ ecdh_validate_key(rng_t *rng, const pgp_ec_key_t *key, bool secret)
     }
 
     if (!ecdh_load_secret_key(&bskey, key) ||
-        botan_privkey_check_key(bskey, rng_handle(rng), 0)) {
+        botan_privkey_check_key(bskey, rng->handle(), 0)) {
         goto done;
     }
     ret = RNP_SUCCESS;
@@ -305,7 +185,7 @@ done:
 }
 
 rnp_result_t
-ecdh_encrypt_pkcs5(rng_t *                  rng,
+ecdh_encrypt_pkcs5(rnp::RNG *               rng,
                    pgp_ecdh_encrypted_t *   out,
                    const uint8_t *const     in,
                    size_t                   in_len,
@@ -323,7 +203,12 @@ ecdh_encrypt_pkcs5(rng_t *                  rng,
     if (!key || !out || !in || (in_len > sizeof(m))) {
         return RNP_ERROR_BAD_PARAMETERS;
     }
-
+#if !defined(ENABLE_SM2)
+    if (key->curve == PGP_CURVE_SM2_P_256) {
+        RNP_LOG("SM2 curve support is disabled.");
+        return RNP_ERROR_NOT_IMPLEMENTED;
+    }
+#endif
     const ec_curve_desc_t *curve_desc = get_curve_desc(key->curve);
     if (!curve_desc) {
         RNP_LOG("unsupported curve");
@@ -347,12 +232,12 @@ ecdh_encrypt_pkcs5(rng_t *                  rng,
     }
 
     if (!strcmp(curve_desc->botan_name, "curve25519")) {
-        if (botan_privkey_create(&eph_prv_key, "Curve25519", "", rng_handle(rng))) {
+        if (botan_privkey_create(&eph_prv_key, "Curve25519", "", rng->handle())) {
             goto end;
         }
     } else {
         if (botan_privkey_create(
-              &eph_prv_key, "ECDH", curve_desc->botan_name, rng_handle(rng))) {
+              &eph_prv_key, "ECDH", curve_desc->botan_name, rng->handle())) {
             goto end;
         }
     }
