@@ -54,8 +54,8 @@ TEST_F(rnp_tests, test_issue_1171_key_import_and_remove)
     assert_rnp_success(rnp_key_get_bits(key, &bits));
     assert_int_equal(bits, 256);
 
-    /* directly use rnp_key_store_get_key_by_grip() which caused crash */
-    pgp_key_t *subkey = rnp_key_store_get_key_by_grip(ffi->pubring, key->pub->grip());
+    /* directly use rnp_tests_get_key_by_grip() which caused crash */
+    pgp_key_t *subkey = rnp_tests_get_key_by_grip(ffi->pubring, key->pub->grip());
     assert_int_equal(subkey->material().bits(), 256);
     assert_rnp_success(rnp_key_handle_destroy(key));
 
