@@ -39,11 +39,13 @@ all_keys_valid(const rnp_key_store_t *keyring, pgp_key_t *except = NULL)
 
     for (auto &key : keyring->keys) {
         if ((!key.valid() || key.expired()) && (&key != except)) {
-            assert_true(rnp::hex_encode(key.keyid().data(),
-                                        key.keyid().size(),
-                                        keyid,
-                                        sizeof(keyid),
-                                        rnp::HEX_LOWERCASE));
+            if (!rnp::hex_encode(key.keyid().data(),
+                                 key.keyid().size(),
+                                 keyid,
+                                 sizeof(keyid),
+                                 rnp::HEX_LOWERCASE)) {
+                throw std::exception();
+            }
             RNP_LOG("key %s is not valid", keyid);
             return false;
         }
